package drakfirsttime::firstboot;

use strict;
use lib qw(/usr/lib/libDrakX /usr/lib/libDrakX/drakfirsttime);
use standalone;
use common;
use any;
use lang;

BEGIN { unshift @::textdomains, 'drakfirstboot' }

use ugtk2 qw(:all :create);
use mygtk2 qw(gtknew);
use data;
use comdialog;
use SOAP::Lite;
# For debugging purpose
# use Data::Dumper;

my %countries = map { lang::c2name($_) => $_ } lang::list_countries(exclude_non_installed => 1);

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $this = bless({
		      version => "1.3",
		      drakfirstboot => N("First Boot Wizard") ,
		      mdkclub => N("Mandriva Club Registration"),
		      pp => N("First Boot Wizard"),
		      sysconffile => '/etc/sysconfig/firstboot',
		      systemfile => '/etc/sysconfig/system',
		      uri => 'https://my.mandriva.com/soap/',
		      serviceProxy => 'https://my.mandriva.com/soap/'
		     }, $class);
    my $rc = "/usr/share/drakfirsttime/themes-galaxy.rc";
    Gtk2::Rc->parse($rc);
    add_icon_path("/usr/share/drakfirsttime/pixmaps/");
    my $default_bg = "/usr/share/mdk/backgrounds/default.png";
    Gtk2->init;
    $this->initialize(@_);
    return;
}
sub initialize {
    my $this = shift;
    $this->{is_network} = network_enabled();
    $this->{is_download} = mdk_downloaded();
    $this->get_country;
    my $prog_name = $this->{is_download} ? 'download' : 'pp';
    $this->{mw} = ugtk2->new(N("First Boot Wizard"), center => 1);

    # We've no WM and the logo background window is stealing the
    # focus when the mouse is not over us, which is not user-friendly:
    mygtk2::_force_keyboard_focus($this->{mw}->{real_window});
	     
    if ($this->get_firstboot_status eq 'yes') { 
	$this->{firstboot} = 1;
	$this->set_ornaments;
	gtkset_size_request($this->{mw}{window}, $::real_windowwidth, $::real_windowheight);
	$this->{mw}{window}->move(
         lang::text_direction_rtl() ? 30 : $::rootwidth - ($::windowwidth + $::real_windowwidth) / 2 -15,
         60 + ($::windowheight - $::real_windowheight) / 2);
    } else {
	gtkset_size_request($this->{mw}{window}, 576, 470);
    }
    $this->{nb} = Gtk2::Notebook->new;
    $this->{Button_SkipWiz} = Gtk2::Button->new(N(" Skip Wizard ")); $this->{Button_Prev} = Gtk2::Button->new(translate("Previous")); $this->{Button_Next} = Gtk2::Button->new(translate("Next")); 
    $this->{Button_Priv} = Gtk2::Button->new(N("Privacy"));
    $this->set_clean;
    my $stage = {
		 download => {
			0 => sub { $this->start },
			1 => sub { $this->survey },
			3 => sub { if ($this->{is_network}) { $this->create_generic_account } else { $this->restore_and_quit } },
			4 => sub { $this->send_key },
			5 => sub { $this->set_services }
			    },
		 mdkclub => {
			0 => sub { $this->start },
			1 => sub { $this->create_generic_account },
			2 => sub { Gtk2->main_quit }
		       },
		 pp => {
			0 => sub { $this->start },
			2 => sub { if ($this->{is_network}) { $this->create_generic_account } else { $this->restore_and_quit } },
			3 => sub { $this->send_key },
			4 => sub { $this->set_services }
		       }
		};
    $this->{Button_SkipWiz}->signal_connect(clicked => sub {  $this->restore_and_quit });
    $this->{Button_Prev}->signal_connect(clicked => sub { $this->on_prev_clicked($stage->{$prog_name}) });
    $this->{Button_Next}->signal_connect(clicked => sub { $this->on_next_clicked($stage->{$prog_name}) });
    $this->{Button_Priv}->signal_connect(clicked => sub { $this->privacy_dialog });
    my $hbox_final = gtkpack_(Gtk2::HBox->new(0, 5),
			      0, gtkpack__(create_hbox('start'),
					   $this->{Button_SkipWiz},
					   $this->{Button_Priv},
					  ),
			      1, Gtk2::Label->new(""),
			      0, gtkpack__(create_hbox('start'),
					   $this->{Button_Prev},
					   $this->{Button_Next},
					  ),
			     );

    gtkadd($this->{mw}{window},
	   gtkpack_(Gtk2::VBox->new(0, 2),
		    1, gtkset_border_width(gtkadd($this->build_ui($prog_name)), 5),
		    0, Gtk2::HSeparator->new,
		    0, gtkpack($hbox_final),
		   ),
	  );
    $< == 0 and $this->tag_firstboot('no'); $this->{Button_Prev}->set_sensitive(0);
    $this->{Button_Next}->grab_focus;
    $this->{mw}{window}->show_all;
    $this->{mw}->main;
    ugtk2->exit(0);
}
sub set_ornaments {
    my $this = shift;
    # Let's save former screen dimension before altering resolution
    ($::genrootwidth, $::genrootheight) = (Gtk2::Gdk->screen_width, Gtk2::Gdk->screen_height);
    $this->xrandr(800,600);
    $this->init_sizes;
    $this->create_logo_window;
    $this->create_steps_window;
}
sub create_logo_window {
    my $this = shift;
    return if $::logowidth == 0;
    my $file = "logo-mandriva.png";
    $this->{logo_window} =
      gtknew('Window',
             width => $::logowidth, height => $::logoheight,
             position => [0, 0],
	     widget_name => 'logo',
             child => $this->{fixed} = Gtk2::Fixed->new,
	    );
    $this->{fixed}->set_has_window(1);
    $this->{fixed}->put(gtkshow(gtknew('Image', file => $file)), 0, 0);
    $this->{logo_window}->signal_connect(realize => sub {
                                             return if !$_[0]->realized;
                                             ugtk2::set_back_pixbuf($this->{logo_window}, gtkcreate_pixbuf($file));
                                         });
    $this->{logo_window}->show;
}
my %steps;
my @WizardsSteps;
sub create_steps_window {
    my $this = shift;
    @WizardsSteps = (
		     welcome       => [ N_("Welcome"), 1, 1, '', '' ],
		     questionnaire  => [ N_("Questionnaire"), 1, -1, '', '' ],
		     isAccount     => [ N_("Mandriva Account"), 1, 1, '1', '' ],
		     productKey   =>  [ N_("Key Registration"), 1, 0, '1', '' ],
		     summary       => [ N_("Summary"), 1, 0, '', ],
		    );
    
    return if $::stepswidth == 0;
    
    $steps{$_} ||= gtknew('Pixbuf', file => "steps_$_") foreach qw(on off done);
    my $category = sub { 
        gtknew('HBox', children_tight => [ 
					  gtknew('Label', text_markup => '<b><u>' . $_[0] . '</u></b>', widget_name => 'Step-categories')
					 ]);
    };
        
    my @l = $category->(N("First Boot Wizard"));
    for (my $i = 0; $i < @WizardsSteps; $i += 2) {
	my $img = gtknew('Image', file => 'steps_off.png');
	$steps{steps}{$i}{img} = $img;
	$steps{steps}{$_}{raw_text} = translate($WizardsSteps[$i + 1]->[0]);
	push @l, gtknew('HBox', spacing => 7, children_tight => [ '', '', $img, $steps{steps}{$_}{text} = gtknew('Label', text => $steps{steps}{$_}{raw_text}) ]);
	#	push @l, gtknew('HBox', spacing => 7, children_tight => [ '', '', $img, translate($WizardsSteps[$i + 1]->[0]) ]);
    }
    
    my $offset = 25;
    $this->{fixed}->put(gtkshow(gtknew('VBox', spacing => 6, children_tight => \@l)),
                        lang::text_direction_rtl() ? $::rootwidth - $::stepswidth : $offset, 150);

    $steps{steps}{0}{img}->set_from_pixbuf($steps{on});
}
sub update_steps_position {
    my $this = shift;
    return if !$steps{steps};
    my $last_step;
    foreach (@WizardsSteps) {
        exists $steps{steps}{$_} or next;
        if ($this->{steps}{$_}{entered} && !$this->{steps}{$_}{done}) {
            $steps{steps}{$_}{img}->set_from_pixbuf($steps{on});
            $steps{steps}{$_}{text}->set_markup('<b><i>' . $steps{steps}{$_}{raw_text} . '</i></b>');
            if ($last_step) {
             $steps{steps}{$last_step}{img}->set_from_pixbuf($steps{done});
             $steps{steps}{$last_step}{text}->set_markup('<b>' . $steps{steps}{$last_step}{raw_text} . '</b>');
         }  
            return;
        }
        $last_step = $_;
    }
}
sub init_sizes {
    my $this = shift;
    ($::rootwidth, $::rootheight) = (800, 600);
    $::stepswidth = $::rootwidth <= 640 ? 0 : 220;
    ($::logowidth, $::logoheight) = $::rootwidth <= 640 ? (0, 0) : (800, 600);
    ($::windowwidth, $::windowheight) = ($::rootwidth - $::stepswidth, $::rootheight - 75);
    ($::real_windowwidth, $::real_windowheight) = (500, 370);
}
sub start { my $this = shift; $this->setFirstPage(0); $this->{nb}->next_page }
sub build_ui {
    my $this = shift;
    my $name = shift;
    my $prog = {
		download => sub { $this->drakfirsttime_wizsteps },
		mdkclub => sub { $this->mdkclub_wizsteps },
		pp => sub { $this->drakfirsttime_wizsteps }
	       };
    $prog->{$name}->();
}
sub set_clean {
    my $this = shift;
    $this->{nb}->set_show_tabs(0); $this->{nb}->set_show_border(0);
}
sub final_page {
    my $this = shift;
    gtkappend_page($this->{nb},
		   gtkpack_(Gtk2::VBox->new(0, 3),
			    1, create_scrolled_window(
						      gtktext_insert(Gtk2::TextView->new,
								     N("Congratulations!

Your Mandriva account has been set up and configured. 
Your rights to Mandriva web services have now taken effects. By clicking on \"Finish\", a club package source will be added if you are eligible and your configuration will be uploaded to take advantage of Mandriva Online services.

Please, visit the links below and sign in to check your current settings and services available:

- http://www.mandrivaonline.com
- http://www.mandrivaclub.com
- http://www.mandrivaexpert.com

")
								    ),
						      [ 'never', 'automatic' ],
						     )))
}
sub welcome {
    my ($this, $class) = @_;
    my $mode = 1;
    my $text = $class eq 'dl' ? N_("Hi,\n

You installed Mandriva Free, the free to download and use version of Mandriva Linux.

This product represents one year of full-time work for Mandriva engineers and the worldwide community of Mandriva Linux contributors.

At Mandriva, we believe in the Open Source model and fully adhere to the GPL and everything it implies. That's why we pride ourselves in keeping this product available to all.

However, to further improve our products, we ask you to kindly spend a few minutes of your time filling out the following survey. This information will help us provide you with a product that's ever more innovating, user-friendly AND tailored to your needs.

We guarantee that your answers will be kept fully private. None of the information you provide will be communicated to third parties.

If you do not wish to answer those few questions, please check the \"Do not send this questionnaire\" box on the next page.

**********
We wish to thank you for the time you spent filling out this survey.") : 
  N_("Hi,

Mandriva Firsttime is an assistant which will help you to register your pack easily in order to benefit from all services included with your Mandriva Linux pack.


Take advantage of free time-limited membership to:
     
     - Mandriva Expert for receiving assistance
     - Mandriva Club for joining the Mandriva Linux Community
     - Mandriva Online for keeping your computer up-to-date


For more details about these services, consult the flyer included with your pack or visit http://www.mandriva.com");
    gtkappend_page($this->{nb},
		   gtkpack_(Gtk2::VBox->new(0, 3),
#			    0, set_darea(N("Welcome to Firstboot"), 'welcome'),
			    1, create_scrolled_window(gtktext_insert(Gtk2::TextView->new, translate($text))),
			   ));
}
sub drakfirsttime_wizsteps {
    my $this = shift;
    if (mdk_downloaded()) {
	$this->welcome('dl');
	$this->question;
    } else { 
	$this->welcome('pp');
    }
    $this->account_chooser;
    $this->provide_key;
    $this->final_page;
    $this->{nb};
}
sub mdkclub_wizsteps {
    my $this = shift;
    $this->set_clean(N("Mandriva Account"), N("\nThis assistant will help you to subscribe to Mandriva Web Services,\n"));
    network_enabled() and $this->account_chooser;
    $this->set_clean(N("Thank You"), N("You will have more information about the club community at\nhttp://www.mandrivaclub.com.\n"));
    $this->{nb};
}
sub mdk_downloaded {
    my $isdown = get_vars_from_hash('/etc/sysconfig/system', 'META_CLASS') eq 'download' ? 1: 0;
    $isdown;
}
sub display_entry {
    my $this = shift;
    my @t;
    foreach my $l (qw(ent1 ent2 ent3 ent4 ent5)) {
	$this->{$l} = Gtk2::Entry->new_with_max_length(7);
	$this->{$l}->set_size_request(50,25);
	push(@t, $this->{$l});
	push(@t, Gtk2::Label->new("-"));
    };
    pop(@t);
    @t
}
my @greets = (N("Mr."), N("Mrs."), N("Ms."));
sub account_chooser {
    my $this = shift;
    $this->{newmember} = Gtk2::RadioButton->new_with_label(undef, N("Create a Mandriva Account"));
    $this->{member} = Gtk2::RadioButton->new_with_label_from_widget($this->{newmember}, N("I already have a Mandriva Account"));
    $this->{nothanks} = Gtk2::RadioButton->new_with_label_from_widget($this->{newmember}, N("No, thanks"));
    my $nb_club = Gtk2::Notebook->new; $nb_club->set_show_tabs(0); $nb_club->set_show_border(0);
    gtkappend_page($this->{nb},
		   gtkpack_(Gtk2::VBox->new(0, 3),
			    1, create_scrolled_window(
						      #Gtk2::WrappedLabel->new(formatAlaTeX(
						      gtktext_insert(Gtk2::TextView->new,
								     N("Mandriva Account!

Mandriva Account is designed to allow you to use a single set of authentication credentials to access all of the secure Mandriva Web services and applications to which you have entitlement rights:

 * Mandriva Online - Easy updates
 * Mandriva Club - Mandriva user community  
 * Mandriva Expert - Online support 
 * Mandriva Store - Online store
 - ...

You can use any existing e-mail address from any e-mail provider as a login to sign in any Mandriva website.
")
								    ),
						      [ 'never', 'automatic' ],
						     ),
			    0, gtkadd(Gtk2::Frame->new(N("Sign Up")),
				      gtkpack_(Gtk2::VBox->new(0,0),
					       map { my $t = $_->[0]; my $s = $_->[1];
						     0, gtksignal_connect($this->{$t}, clicked => sub { 
									      if ($t eq 'nothanks') {
										  $this->{$t}->get_active and $this->ask_before_exit
									      } else { $nb_club->set_current_page($s) }
									  })
						 } (['newmember', 0], ['member', 1], ['nothanks', 2])
					      )))
		  );
    gtkappend_page($this->{nb},
		   gtkpack_(Gtk2::VBox->new(0,3),
			    1, $nb_club
			   )
		  );
    gtkappend_page($nb_club,
		   gtkpack_(Gtk2::VBox->new(0, 2),
			    0, gtkadd(gtkset_shadow_type($this->{gfp} = Gtk2::Frame->new(N("Create a Mandriva Account")), 'etched_in'),
				      gtkpack_(Gtk2::VBox->new,
					       1, gtkset_border_width(create_packtable({ col_spacings => 10, row_spacings => 10 },
								   [ N("Greeting:"), $this->{greeting} = combo_sized(@greets) ],
								   [ N("First name:"), $this->{firstname} = entry_sized() ],
								   [ N("Last name:"), $this->{lastname} = entry_sized() ],
								   [ N("Email address:"), $this->{account} = entry_sized() ],
								   [ N("Password:"), $this->{newpasswd} = entry_hidepass() ],
								   [ N("Confirm Password:"), $this->{confirm} = entry_hidepass() ],
								  ),
								     8),
					      )))
		  );
    gtkappend_page($nb_club,
		   gtkpack_(Gtk2::VBox->new(0, 2),
			    0, gtkadd(gtkset_shadow_type($this->{gfp} = Gtk2::Frame->new(N("Mandriva Registration Update")), 'etched_in'),
				      gtkpack_(Gtk2::VBox->new(0, 5),
					       1, gtkset_border_width(create_packtable({ col_spacings => 10, row_spacings => 10 },
								   [], [],
								   [ N("Email address:"), $this->{login} = Gtk2::Entry->new ],
										       [ N("Password:"), $this->{passwd} = entry_hidepass() ],
								       ),
								  	8),
					      )))
		  );
}
sub provide_key {
    my $this = shift;
    my $mode = 1; my $hbox;
    gtkappend_page($this->{nb},
		   gtkpack_(Gtk2::VBox->new(0, 3),
			    1, create_scrolled_window(
						      #Gtk2::WrappedLabel->new(formatAlaTeX(
						      gtktext_insert(Gtk2::TextView->new,
								     N("Register Your Pack!

If you are an owner of a Mandriva packaged product, you are entitled to many benefits including product support, free months of Mandriva Club and Mandriva Online memberships. 

Enter your registration key below and Click on Next to see a summary of entitled services.
")
								    ),
						      [ 'never', 'automatic' ],
						     ),
			    0, gtkadd(Gtk2::Frame->new(N("Register key")),
				      gtkpack_(Gtk2::VBox->new(0,0),
					       0, gtksignal_connect(gtkset_active($this->{donotsend} = Gtk2::CheckButton->new(N("I do not have a product key")), !$mode),
								    clicked => sub { $mode = !$mode; $hbox->set_sensitive($mode) }),
					       0, gtkpack($hbox = Gtk2::HBox->new(0, 0), $this->display_entry)
					      )
				     )
			   )
		  );
}
sub privacy_dialog {
    my $this = shift;
    my $d = ugtk2->new(N("Privacy Text"), grab => 1, transient => $this->{mw}{window});
    $d->{window}->set_position('center_on_parent');
    gtkadd(
      $d->{window},
	   gtkpack_( Gtk2::VBox->new,
		     1, ( gtkadd(
				 gtkset_shadow_type(Gtk2::Frame->new, 'in'),
				 gtkset_size_request(create_scrolled_window(gtktext_insert(Gtk2::TextView->new, privacy_text())), 580, 440),
				)
			),      
		     0, gtkpack( create_hbox(),
				 gtksignal_connect(Gtk2::Button->new(N("Ok")), clicked => sub { Gtk2->main_quit })
			       )
		   )
	  );
    $d->{rwindow}->show_all;
    $d->main
}
sub question {
    my $this = shift;
    my $mode = 1;
    my %quests = (
		  a => { wg => [ N("Yes"), N("No") ], 
			 quest => N("Is this your first Linux system?") },
		  b => { wg => [ 'Windows(R)', 'MacOS(R)', 'Unix(R)', 'Linux(R)' ], 
			 quest => N("What is your current operating system?") },
		  c => { wg => [ N("Yes"), N("No") ],
			 quest => N("Is this your first Mandriva Linux distribution?") },
		  d => { wg => [ 'RedHat', 'SUSE', 'Debian', 'Ubuntu', 'Gentoo', 'Linspire', 'Turbo Linux', N("Other") ], 
			 quest => N("What is your current operating system?") },
		  f => { wg => [ N("At Home"), N("In a business"), N("Both") ],
			 quest => N("Where are you using this distribution?") },
		  g => { wg => [ N("Very small ( < 10)"), N("Small ( < 50)"), N("Medium (50 to 500)"), N("Large (> 500)") ],
		         quest => N("What is the size of your organisation?") },
		  h => { wg => [ '1', '1-5', '5-10', '10-50', N("Over 50") ],
			 quest => N("On how many computers do you plan to install it?") },
		  i => { wg => [ N("Ease of use"), N("Integrated Desktop"), N("Performance"), 
				 N("Stability"), N("Innovations"), N("Compatibility") ],
			 quest =>  N("What do you think is the most appealing feature in Mandriva Linux?") },
		  j => { wg => [ 'KDE', 'GNOME', 'IceWM', 'WindowMaker', N("Other") ], 
			 quest => N("Which Linux graphical environment do you prefer?") },
		  k => { wg => [ 'Kontact', 'Evolution', 'Thunderbird', N("Other") ],
			 quest => N("Which PIM (Personal Information Manager) do you prefer?") },
		  l => { wg => [ 'Konqueror', 'Mozilla', 'Mozilla Firefox', 'Epiphany', N("Other") ],
			 quest => N("Which web browser do you prefer?") },
		  n => { wg => [ N("Desktop"), N("Server"), N("Desktop and server(s)") ],
			 quest => N("For what usage do you plan to use this distribution?") },
		  o => { wg => [ N("A Workstation"), N("A Laptop"), N("Both") ],
			 quest => N("Do you use?") },
		  p => { wg => [ N("Office"), N("Multimedia"), N("Internet"), N("Development tools"), N("Server") ],
			 quest => N("Which function do you use most of the time?")
		       },
		  r => { wg => [ N("You are a faithful Mandriva Linux user"), N("You are a Linux user"), N("You want to test Linux") ],
			 quest => N("Which of the following best describes you?") },
		  t => { wg => [ N("Under 15"), '15-25', '25-35', '35-45', '45-55', N("over 55") ],
			 quest => N("How old are you?") },
		  u => { wg => [ N("Male"), N("Female") ], 
			 quest => N("Are you?") },
		  v => { wg => [ N("Yes"), N("No") ],
			 quest => N("Do you allow uploading information about your hardware settings?") },
		  w => { wg => [ N("Yes"), N("No") ],
			 quest => N("Would you like to receive the Mandriva Linux News?") },
		 );
    my %q;
    my %sense = ( a1 => ['b', 'c'], a2 => ['c', 'b'], c1 => ['d', ''], c2 => ['', 'd'], f1 => ['', 'g'], f2 => ['g', ''], f3 => [ 'g', ''], w1=> ['email', ''], w2 => ['', 'email'] );
    foreach my $st ( keys %quests ) {
	my $inval = $st . 'x';
	$q{$st . 'x'} = Gtk2::RadioButton->new_with_label(undef, '');
	$this->{$st}{val} = $inval;
	foreach my $num (1..6) {
	    if (defined $quests{$st}{wg}->[$num-1]) {
		my $val = $st . $num;
		$q{$val} = Gtk2::RadioButton->new_with_label_from_widget($q{$inval}, $quests{$st}{wg}->[$num-1]);
		$q{$val}->signal_connect(clicked => sub { 
			     $this->{$st}{val} = $val;
			         if (member($val, keys %sense)) {
				     if (defined $this->{$sense{$val}->[0]}{box}) {
					 $this->{$sense{$val}->[0]}{box}->set_sensitive(1) 
				     }
				     if (defined $this->{$sense{$val}->[1]}{box}) { 
					 $this->{$sense{$val}->[1]}{box}->set_sensitive(0);
					 $this->{$sense{$val}->[1]}{val} = $sense{$val}->[1] . 'x'
				     }
				 }
			 });
	    }
	}
	gtkpack($this->{$st}{box} = Gtk2::VBox->new(0, 0),
		gtkpack_(Gtk2::HBox->new(0,0), 0, Gtk2::Label->new($quests{$st}{quest}), 1, Gtk2::Label->new(" ")),
		create_packtable({ homogeneous => 0, col_spacings => 20, row_spacings => 0 },
				 map_index { $quests{$st}{wg}->[$::i] && [ '', $q{$st . ($::i+1)} ] } (0..5)
				));
    }
    $q{v1}->set_active(1); $this->{v}{val} = 'v1';
    gtkpack_($this->{email}{box} = Gtk2::HBox->new(0, 0),
	     0, Gtk2::Label->new(N("Email") . ':'), 1, $this->{dwnemail} = entry_sized() );
    gtkpack_($this->{country}{box} = Gtk2::HBox->new(0, 0), 
	     0, Gtk2::Label->new(N("Country") . ':'), 1, $this->{dwncountry} = Gtk2::OptionMenu->new);
    $this->{dwncountry}->set_popdown_strings(sort(keys %countries));
    $this->{dwncountry}->set_text($this->{cnt});
    map { defined $this->{$_->[0]}{box} and $this->{$_->[0]}{box}->set_sensitive(0) } values %sense;
    gtkappend_page($this->{nb},
		   gtkpack_(Gtk2::VBox->new(0, 0),
			    1, gtkpack_(Gtk2::VBox->new(0, 0),
					0, gtksignal_connect(gtkset_active($this->{donotsend} = Gtk2::CheckButton->new(N("Do not send this questionnaire")), !$mode),
							     clicked => sub { $mode = !$mode; $this->{vb}->set_sensitive($mode) }),
					1, gtkpack_($this->{vb} = Gtk2::VBox->new(0,0),
						    1, create_scrolled_window( gtkpack__(Gtk2::VBox->new(0,0), map { $_ ne 'm' and $this->{$_}{box} } ('a'..'w', 'email', 'country'),
											))))));
}
sub grayed {
    my $this = shift;
    my $c = $this->{mailclient}->entry->get_text ? 1 : 0;
    foreach (qw(vbox server frame)) {
	$this->{$_}->set_sensitive($c);
    }
}
sub get_lang() {
    my $locale = lang::read('', $>);
    my $lang = $locale->{lang};
    $lang
}
sub get_country {
    my $this = shift;
    $this->{cnt} = lang::c2name(lang::read('', $>)->{country});
}
sub survey {
    my $this = shift;
    my ($resp, $send, $arch, $mail, $lang, $locale);
    my %add;
    my $netconf = '/etc/sysconfig/network-scripts/ifup.d/survey.pl';
    my $link = 'http://poll.mandriva.com/online_survey.php';
    my ($r) = cat_('/etc/release') =~ /release\s(\S+)/; 
    $arch = chomp_(`/bin/arch`);
    my $lang = get_lang();
    foreach my $m (qw(dwnemail dwncountry)) { $this->{$m} and $add{$m} = $m eq 'dwnemail' ? $this->{$m}->get_text : $this->{$m}->entry->get_text };
    map { $resp .= $this->{$_}{val} } ('a'..'w'); 
    my $ct = lc($countries{$add{dwncountry}});
    $mail = $add{dwnemail};
    if($this->{is_network} && !$this->{sent}) {
	$send = comdialog::request_URL($link . '?rep=' . $resp . '&ver=' . $r . '&arch=' . $arch . '&lang=' . $lang . '&email='. $mail . '&country_id=' . $ct);
	$this->{sent} = 1;
    }
    if ($this->{v}{val} eq 'v1') {
	my $meta_class = get_vars_from_hash('/etc/sysconfig/system', 'META_CLASS');
	$r =~ s/\.//;
	my $db_login = 'mdk' . $r;
	system("HWDB_PASSWD=$db_login hwdb_add_system $db_login $meta_class &")
    }
    $this->{nb}->next_page;
}
sub create_generic_account {
    my $this = shift;
    #$this->{productkey} = '';
    #map { $this->{productkey} .= $this->{$_}->get_text } qw (ent1 ent2 ent3 ent4 ent5); 
    my ($ua, $code, $info);
    $ua = get_ver_rel('drakfirsttime');
    $this->{soap} = is_proxy() ? SOAP::Lite->uri($this->{uri})->proxy($this->{serviceProxy}, proxy => [ 'http' => $ENV{http_proxy} ], agent => $ua) : SOAP::Lite->uri($this->{uri})->proxy($this->{serviceProxy}, agent => $ua);
    my $hreturn = {
		   1  => [ N("Security error"), N("Unsecure invocation: Method available through httpS only") ],
		   2  => [ N("Database error"), N("Server Database failed\nPlease Try again Later") ],
		   3  => [ N("Registration error"), N("Some parameters are missing") ],
		   5  => [ N("Password error"), N("Wrong password") ],
		   7  => [ N("Login error"), N("The email you provided is already in use\nPlease enter another one\n") ],
		   8  => [ N("Login error"), N("The email you provided is invalid or forbidden") ],
		  10  => [ N("Login error"), N("Email address box is empty\nPlease provide one") ],
		  12  => [ N("Restriction Error"), N("Database access forbidden") ],
		  13  => [ N("Service error"), N("Mandriva web services are currently unavailable\nPlease Try again Later") ],
		  17  => [ N("Password error"), N("Password mismatch") ],
		  20  => [ N("Service error"), N("Mandriva web services are under maintenance\nPlease Try again Later") ],
		  22  => [ N("User Forbidden"), N("User account forbidden by Mandriva web services") ], 
		  99  => [ N("Connection error"), N("Mandriva web services not reachable") ]
		  };
    foreach my $num ([9, 8], [21, 20]) { $hreturn->{$num->[0]} = $hreturn->{$num->[1]} };
    my $callback = {
		    newmember => sub { 
			if ($this->{newpasswd}->get_text ne $this->{confirm}->get_text) { 
			    raiseError($this->{mw}, N("Mismatch"), N("The passwords do not match\n Please try again\n"))
			} elsif (check_valid_email($this->{account}) != 1) {
			    raiseError($this->{mw}, N("Invalid Mail"), N("Not a valid mail address!\n"))
			} else {
			    @$info = map { $this->{$_}->get_text } qw(account newpasswd firstname lastname);
			    if (check_empty_field(@$info)) {
				raiseError($this->{mw}, N("Empty parameters"), N("Please fill in each field"));
			    } else {
				my $lg = get_lang();
				my $gr = $this->{greeting}->entry->get_text;
				$gr = find_index { /$gr/ } @greets;
				#AccountType = home, alias = firstname + lastname
				my $alias = $this->{firstname}->get_text . $this->{lastname}->get_text;
				push(@$info, $lg, $gr, $alias, 'home');
				eval { $this->{response} = $this->soap_create_account($info) };
				# print Dumper($info);
				# print Dumper($this->{response});
				if ($this->{response}{status}) {
				    $this->{log} = $this->{response}{data}->{email};
				    $this->{pass} = $this->{newpasswd}->get_text();
				    $this->{nb}->next_page;
				} else {
				    $code = $this->{response}{code} || '99';
				    raiseError($this->{mw}, $hreturn->{$code}->[0], $hreturn->{$code}->[1]) 
				}
			    }
			}
		    },
		    member => sub { 
			@$info = map { $this->{$_}->get_text } qw(login passwd);
			if (check_empty_field(@$info)) {
			    raiseError($this->{mw}, N("Empty parameters"), N("Please fill in each field"));
			} else {
			    eval { $this->{response} = $this->soap_authenticate_user($info) };
			    # print Dumper($this->{response});
			    if ($this->{response}{status}) {
				$this->{log} = $this->{response}{data}->{email};
				$this->{pass} = $this->{passwd}->get_text();
				$this->{nb}->next_page;
			    } else {
				$code = $this->{response}{code} || '99';
				raiseError($this->{mw}, $hreturn->{$code}->[0], $hreturn->{$code}->[1]) 
			    }
			}
		    },
		   }; 
    map {
	$this->{$_}->get_active and $callback->{$_}->();
    } keys %$callback;
}
sub send_key {
    my $this = shift;
    my $info;
    $this->{productkey} = uc(join('-', map { $this->{$_}->get_text } qw(ent1 ent2 ent3 ent4 ent5)));
    @$info = map { $this->{$_} } qw(log pass productkey);
    eval { $this->{entitled} = $this->soap_register_key($info) };
    $this->{nb}->next_page;
    #    Dumper($this->{entitled});
}
sub set_services {
    my $this = shift;
    eval { $this->read_mdkonline_conf };
    my ($log, $pass, $machine, $country) = map { $this->{$_} } qw(log pass machine cnt);
    $this->{cnt} ||= 'us';
    eval { $this->AddUrpmiSource() };
    system("mdkonline --nointeractive --login $log --pass $pass --box $machine --country $country &");
    $this->restore_and_quit();
}
sub read_mdkonline_conf {
    my $this = shift;
    my $onlineconf = '/etc/sysconfig/mdkonline';
    my %t = getVarsFromSh($onlineconf);
    my $host = chomp_(`hostname`);
    $t{MACHINE} ||= $1 if $host =~ /(\w+)(.*)/;
    $this->{machine} = $t{MACHINE};
}
sub AddUrpmiSource {
    my $this = shift;
    my $arch = chomp_(`/bin/arch`); $arch =~ /i386|i486|i686$/ and $arch = 'i586';
    my ($r) = cat_('/etc/release') =~ /release\s(\S+)/;
    
    # TODO query mirrors API instead, once it is ready
    my $server = "dl.mandriva.com/rpm/";
    my @sources = (
        'club/' . $r . '/' . $arch,
        'club/' . $r . '/noarch',
        'comm/' . $r . '/' . $arch,
        'comm/' . $r . '/noarch'
    );
    foreach my $source (@sources) {
        my $login = $this->{log};
        my $pass  = $this->{pass};
        $login =~ s/@/%40/g;
        $pass  =~ s/@/%40/g;
        my $link   = "https://" . $login . ':' . $pass . '@' . $server . $source . '/';
        $source =~ s/\//_/g;
        my $cmd = "/usr/sbin/urpmi.addmedia --curl \"" . $source . "\" " . $link . " with hdlist.cz";
        eval { system $cmd };
    }
}
sub ask_before_exit {
    my $this = shift;
    $this->{yesorno} = $this->gtk_msg_(N("Warning"), N("Click on \"Yes\" to skip the wizard"), yesno => 1);
    $this->{yesorno} and $this->restore_and_quit, return;
    $this->{newmember}->set_active(1);
}
sub wait_msg {
    my ($msg, %options) = @_;
    my $mainw = ugtk2->new('club', if_(exists $options{transient}, transient => $options{transient}));
    $mainw->{window}->set_position($options{transient} ? 'center_on_parent' : 'center_always') if !$::isEmbedded;
    my $label = ref($msg) =~ /^Gtk/ ? $msg : Gtk2::Label->new($msg);
    gtkadd($mainw->{window}, gtkpack__(gtkadd(Gtk2::VBox->new(0, 5), $label, if_(exists $options{widgets}, @{$options{widgets}}))));
    $label->signal_connect(expose_event => sub { $mainw->{displayed} = 1; 0 });
    $mainw->sync until $mainw->{displayed};
    gtkset_mousecursor_wait($mainw->{rwindow}->window);
    $mainw->flush;
    $mainw;
}
sub remove_wait_msg { $_[0]->destroy }
sub gtk_msg_ { my $this = shift; gtk_msg(@_, if_(exists $this->{mw}->{rwindow}, transient => $this->{mw}->{rwindow}), centermove => 1) }
sub get2final {
    my $this = shift;
    $this->{Button_Prev}->hide; $this->{Button_SkipWiz}->hide;
    $this->{Button_Next}->set_label(N("Finish"));
}
sub display_last_page {
    my $this = shift;
    $this->get2final;
    $this->{nb}->next_page;
}
sub on_next_clicked {
    my ($this, $stage) = @_;
    $this->{Button_Prev}->set_sensitive(1);
    my $page = $this->{nb}->get_current_page;
    $page == 4 and $this->get2final;
    !$this->{is_download} && $page == 3 and $this->get2final; 
    if (exists($stage->{$page})) { $stage->{$page}->() } else { $this->{nb}->next_page };
}
sub on_prev_clicked {
    my ($this, $stage) = @_;
    my $page = $this->{nb}->get_current_page;
    $this->{Button_Next}->get_label eq N("Finish") and $this->{Button_Next}->set_label(N("  Next  > "));
    $this->{nb}->prev_page;
    $page == 1 and $this->setFirstPage(1)
}
sub setFirstPage {
    my ($this, $state) = @_;
    if (!$state) {
	$this->{Button_Prev}->set_sensitive(1);
	$this->{Button_Priv}->hide;	
    } else {
	$this->{Button_Prev}->set_sensitive(0);
	$this->{Button_Priv}->show
    }
}
sub tag_firstboot { 
    my ($this, $status) = @_;
    output_p $this->{sysconffile}, qq(FIRSTBOOT=$status
)
};
sub restore_and_quit {
    my $this = shift;
    $this->{firstboot} and $this->xrandr($::genrootwidth, $::genrootheight);
    Gtk2->main_quit;
    ugtk2::exit(0);
}
sub xrandr {
    my ($this, $x, $y) = @_;
    my $res = $x . 'x' . $y;
    system ("xrandr --size $res");
}
sub get_firstboot_status {
    my $this = shift;
    my %h = getVarsFromSh($this->{sysconffile});
    $h{FIRSTBOOT};
}
sub is_proxy () {
    return 1 if defined $ENV{http_proxy};
}
sub soap_create_account {
    my ($this, $info) = @_; 
    my $register = $this->{soap}->registerUserFromWizard(@$info)->result();
    $register;
}
sub soap_authenticate_user {
    my ($this, $info) = @_;
    my $auth = $this->{soap}->authenticateUser(@$info)->result(); 
    $auth
}
sub soap_register_key {
    my ($this, $info) = @_;
    my $reg = $this->{soap}->registerKey(@$info)->result(); 
    $reg
}
sub get_ver_rel {
    my $name = shift;
    my $qualified_name = chomp_(`rpm -q $name`);
    $qualified_name
}

1;
